<?php

require_once(__DIR__ . '/Client.php');

class AcronisApi_Auth_v1 implements AcronisApi_AuthInterface
{
    private $_apiUrl;
    private $_User;
    private $_apiClient;

    /**
     * Constructor.
     *
     * @param Acronis_User $User
     */
    public function __construct(Acronis_User $User)
    {
        // get user from plugin - will includes tenant id.
        $Plugin = IW::PluginManager()->getPlugin('acronis');
        $User = $Plugin->getAcronisUser();

        $this->_apiUrl = $User->getDatacenterUrl();
        $this->_apiClient = new AcronisApi_Client($this->_apiUrl);
        $this->_User = $User;
    }

    /**
     * Send authenticated GET request.
     *
     * @param string $apiEndpoint
     * @return array
     */
    public function sendAuthenticatedGetRequest($apiEndpoint)
    {
        $this->_setCookiesIfNeeded();
        $config = $this->_getCookieConfig($apiEndpoint);
        $res = $this->_apiClient->sendGetRequest($config);

        // will throw exception if cookie expired.
        try {
            $this->_validateResponse($res);
            return $this->_returnItems($res);
        } catch (Exception $e) {
            // reissue new cookie.
            $this->_requestBackupConsoleCookie();
            $res = $this->_apiClient->sendGetRequest($config);
            $this->_validateResponse($res);
            return $this->_returnItems($res);
        }
    }

    /**
     * Send authenticated POST request.
     *
     * @param string $apiEndpoint
     * @param array  $data
     * @return array
     */
    public function sendAuthenticatedPostRequest($apiEndpoint, array $data)
    {
        $this->_setCookiesIfNeeded();
        $config = $this->_getCookieConfig($apiEndpoint, $data);
        $res = $this->_apiClient->sendPostRequest($config);

        try {
            $this->_validateResponse($res);
            return $this->_returnItems($res);
        } catch (Exception $e) {
            // reissue new cookie.
            $this->_requestBackupConsoleCookie();
            $res = $this->_apiClient->sendPostRequest($config);
            $this->_validateResponse($res);
            return $this->_returnItems($res);
        }
    }

    /**
     * Send authenticated PUT request.
     *
     * @param string $apiEndpoint
     * @param array  $data
     * @return array
     */
    public function sendAuthenticatedPutRequest($apiEndpoint, array $data)
    {
        $this->_setCookiesIfNeeded();
        $config = $this->_getCookieConfig($apiEndpoint, $data);
        $res = $this->_apiClient->sendPutRequest($config);

        try {
            $this->_validateResponse($res);
            return $this->_returnItems($res);
        } catch (Exception $e) {
            // reissue new cookie.
            $this->_requestBackupConsoleCookie();
            $res = $this->_apiClient->sendPutRequest($config);
            $this->_validateResponse($res);
            return $this->_returnItems($res);
        }
    }

    /**
     * Send request to receive & save cookies if cookies file is missing.
     */
    private function _setCookiesIfNeeded()
    {
        // Will only detect if no cookies have been set yet.
        // Does not detect expired cookies.
        if (!$this->_apiClient->sessionSet()) {
            $this->_requestBackupConsoleCookie();
        }
    }

    /**
     * Get config array for cookie request.
     *
     * @param string $apiEndpoint
     * @param array  $data
     * @return array
     */
    private function _getCookieConfig($apiEndpoint, array $data = null)
    {
        $config = array(
            'params' => $apiEndpoint,
            'set_cookies' => true,
            'headers' => array(
                'Accept' => 'application/json',
                'Content-type' => 'application/json'
            )
        );

        if ($data) {
            $config['data'] = $data;
        }

        return $config;
    }

    /**
     * Validate response object
     *
     * @param mixed $response
     * @throws Exception
     */
    private function _validateResponse($response)
    {
        if (is_array($response) && !empty($response['error'])) {
            $responseError = $response['error'];

            $errorMessage = $responseError['message'];
            $responseCode = $responseError['code'];
            throw new Exception("ERROR: Invalid response auth_v1: $responseCode: $errorMessage");
        }
    }

    /**
     * Format response object to return items/data.
     *
     * @param mixed $res
     * @return array
     */
    private function _returnItems($res)
    {
        if (isset($res['data'])) {
            return $res['data'];
        }

        if (isset($res['items'])) {
            return $res['items'];
        }

        return $res;
    }

    /**
     * Save cookies.
     *
     * @throws Exception
     */
    private function _requestBackupConsoleCookie()
    {
        $token = $this->_requestBackupConsoleAccessToken();

        $headers = $this->_basicHeaders();
        $data = array("access_token" => $token);
        $config = array(
            'headers' => $headers,
            'data' => $data,
            'params' => '/bc/api/remote_connection'
        );

        // no point in returning response - curl will save cookies.
        $saveCookies = true;
        $res = $this->_apiClient->sendPostRequest($config, $saveCookies);
        $this->_validateResponse($res);
    }

    /**
     * Request access tken used for cookie authentication.
     *
     * @return string
     */
    private function _requestBackupConsoleAccessToken()
    {
        $tenantId = $this->_User->getTenantId();
        $headers = $this->_basicHeaders();
        $config = array(
            'headers' => $headers,
            'params' => "/api/1/groups/$tenantId/backupconsole"
        );

        $res = $this->_apiClient->sendGetRequest($config);
        $this->_validateResponse($res);
        return $res['token'];
    }

    /**
     * Get headers for basic authentication.
     *
     * @return array
     */
    private function _basicHeaders()
    {
        $basic = $this->_User->getBasicCredentials();
        $headers = array(
            'Authorization' => "Basic $basic",
            'Accept' => 'application/json',
            'Content-type' => 'application/json'
        );

        return $headers;
    }
}
