<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Dec 16, 2019
 */

require_once(__DIR__ . '/DashboardInterface.php');
require_once(__DIR__ . '/Auth_v1.php');

/**
 * Acronis API for dashboard events.
 */
class AcronisApi_Dashboard implements AcronisApi_DashboardInterface
{

    const CACHE_DEVICE = 'device.acronis';
    const CACHE_RESOURCEID = 'resourceid.acronis';
    const CACHE_PLANID = 'planid.acronis';
    const CACHE_PLAN = 'plan.acronis';
    const CACHE_BACKUP_STATUS = 'status.acronis';

    private $_bcApi;
    private $_oauthApi;

    /**
     * Get backup cloud api, v1. Authorizes with cookies instead of oauth token.
     *
     * @return AcronisApi_Auth_v1
     */
    private function _getBcApi()
    {
        if (!isset($this->_bcApi)) {
            $Plugin = $this->_getPlugin();
            $User = $Plugin->getAcronisUser();
            $this->_bcApi =  new AcronisApi_Auth_v1($User);
        }

        return $this->_bcApi;
    }

    /**
     * Get oauth api.
     *
     * @return AcronisApi_Auth
     */
    private function _getOAuthApi()
    {
        if (!isset($this->_oauthApi)) {
            $Plugin = $this->_getPlugin();
            $this->_oauthApi = $Plugin->getAuthApi();
        }

        return $this->_oauthApi;
    }

    /**
     * Get plugin.
     *
     * @return Plugin
     */
    private function _getPlugin()
    {
        return IW::PluginManager()->getPlugin('acronis');
    }

    /**
     * Request plans.
     *
     *
     * @param string $resourceId
     * @return array
     */
    public function getBackupPlans($resourceId)
    {
        $bcApi = $this->_getBcApi();
        $res = $bcApi->sendAuthenticatedGetRequest("/bc/api/ams/backup/plans");

        $plans = $this->_filterPlansByResourceId($res, $resourceId);
        return $plans;
    }

    /**
     * Filter plans by resource id.
     *
     * @param array  $plans
     * @param string $resourceId
     */
    private function _filterPlansByResourceId(array $plans, $resourceId)
    {
        $resourcePlans = array();

        foreach ($plans as $plan) {
            if ($this->planTargetsResource($plan, $resourceId)) {
                $resourcePlans[] = $plan;
            }
        }

        return $resourcePlans;
    }

    /**
     * Check if resource is targetted by backup plan.
     *
     * @param array  $plan
     * @param string $resourceId
     * @return boolean
     */
    public function planTargetsResource(array $plan, $resourceId)
    {
        $inclusions = $plan['target']['inclusions'];

        foreach ($inclusions as $inclusion) {
            if (strpos($inclusion['resource_key'], $resourceId) !== false) {
                return true;
            }
        }

        return false;
    }

    /**
     * Request backup plan.
     *
     * @param string $planId
     * @return array
     */
    public function getBackupPlan($planId)
    {
        $bcApi = $this->_getBcApi();
        $res = $bcApi->sendAuthenticatedGetRequest("/bc/api/ams/backup/plans/$planId");

        if (empty($res)) {
            throw new Exception('Backup plan does not exist!');
        }

        return $res;
    }

    /**
     * Get backup status from Acronis api.
     *
     * @param string $resourceId
     * @return array
     */
    public function getBackupStatus($resourceId)
    {
        // TODO sort out why the above method is doing the same call...
        $param = '/bc/api/ams/protected_resources?resourceIds=' . $resourceId;

        $bcApi = $this->_getBcApi();
        $resources = $bcApi->sendAuthenticatedGetRequest($param);
        // TODO possibly update to get status by plan id once saved/implemented.
        $resourcePlan = $this->_getBackupStatusByResourceId($resources, $resourceId);

        return $resourcePlan;
    }

    /**
     * Get plan status by resource ID.
     *
     * @param array  $resources
     * @param string $resourceId
     * @return mixed
     */
    private function _getBackupStatusByResourceId(array $resources, $resourceId)
    {
        foreach ($resources as $resource) {
            if (stripos($resourceId, $resource['resourceId']) !== false) {
                return $resource;
            }
        }
    }

    /**
     * Get total storage size. Note that resource id is
     * the shortened version of the id, see _getIdFromResourceKey.
     *
     *
     * @param string $resourceId
     */
    public function getStorageSize($resourceId)
    {
        $bcApi = $this->_getBcApi();
        $apiEndpoint = "/bc/api/vault_manager/v1/stats?resourceId=$resourceId&storageType=online";

        $res = $bcApi->sendAuthenticatedGetRequest($apiEndpoint);
        $size = $res['occupied_data_size'];

        return $size;
    }

    /**
     * Get tasks section data from acronis api.
     *
     * @return array
     */
    public function getTasksSectionData()
    {
        return array(
            'in_progress' => 1,
            'queued' => 1
        );
    }

    /**
     * Request device/server specific backup info.
     *
     * @return array
     */
    public function getDeviceInfo()
    {
        $bcApi = $this->_getBcApi();
        $res = $bcApi->sendAuthenticatedGetRequest('/bc/api/ams/resources');

        if (empty($res)) {
            throw new Exception("ERROR: Request unsuccessful - getDeviceInfo");
        }

        $deviceInfo = $this->_getDeviceInfoFromResponse($res);
        return $deviceInfo;
    }

    /**
     * Get device information from response.
     *
     * @param array $responseItems
     * @return array
     */
    private function _getDeviceInfoFromResponse(array $responseItems)
    {
        foreach ($responseItems as $item) {
            if ($this->_itemIsServer($item)) {
                return $item;
            }
        }
    }

    /**
     * Check if item is server.
     *
     * @param array $item
     * @return boolean
     */
    private function _itemIsServer(array $item)
    {
        return $item['server'] && gethostname() === $item['title'];
    }


    /**
     * Get agent data.
     *
     * @return array
     */
    public function getAgentData()
    {
        $bcApi = $this->_getBcApi();
        $res = $bcApi->sendAuthenticatedGetRequest('/api/resource_manager/v1/agents');
        return $res;
    }

    /**
     * Gets redirect url that logs in user.
     *
     * @return string
     */
    public function getLoginUrl()
    {
        $oauthApi = $this->_getOAuthApi();
        $username = $oauthApi->getUsername();
        $config = $this->_getOttConfig($username);

        $response = $oauthApi->sendAuthenticatedRequest($config);
        $this->_handleOttResponseError($response);

        $query = array('ott' => $response['ott']);
        $httpQuery = http_build_query($query);

        $apiUrl = $oauthApi->getApiBaseUrl();
        $redirectUrl =  $apiUrl . "idp/external-login#$httpQuery&targetURI=$this->_url/bc";

        return $redirectUrl;
    }

    /**
     * Handle ott response error.
     *
     * @param array $response
     */
    private function _handleOttResponseError(array $response)
    {
        $responseError = $response['error'];
        if ($responseError) {
            $responseCode = $responseError['code'];
            $responseMessage = $responseError['message'];
            throw new Exception("Ott request failed: $responseCode: $responseMessage");
        }
    }

    /**
     * Get config array for ott request.
     *
     * @param string $username
     * @return array
     */
    private function _getOttConfig($username)
    {
        $ottData = array(
            'login' => $username,
            'purpose' => 'user_login'
        );

        $headers = array(
            'Content-type' => 'application/json',
            'Accept' => 'application/json'
        );

        $config = array(
            'headers' => $headers,
            'data' => $ottData,
            'params' => '/api/2/idp/ott'
        );

        return $config;
    }

    /**
     * Get resource id.
     *
     * @return string
     */
    public function getResourceId()
    {
        $deviceInfo = $this->getDeviceInfo();
        return $deviceInfo['id'];
    }

    /**
     * Get applicable plans.
     *
     * @param string $resourceId
     */
    public function getApplicablePlans($resourceId)
    {
        $bcApi = $this->_getBcApi();
        $apiEndpoint = '/bc/api/ams/backup/plan_draft_operations/get_applicable_plans';
        $data = array(
            'resourcesIds' => [$resourceId]
        );

        $res = $bcApi->sendAuthenticatedPostRequest($apiEndpoint, $data);
        return $res;
    }

    /**
     * Create draft for backup plan.
     *
     * @param string $resourceId
     * @return array
     */
    public function createDraft($resourceId)
    {
        // TODO enable daily backup = defaults to mon-fri
        $defaultStartTime = array(
            'hour' => 2,
            'minute' => 0,
            'second' => 0
        );

        $data = array(
            'language' => 'en',
            'sourceType' => 'machines',
            'bootableMediaPlan' => false,
            'defaultStartTime' => $defaultStartTime,
            'resourcesIds' => array($resourceId)
        );

        $apiEndpoint = '/bc/api/ams/backup/plan_drafts';

        $bcApi = $this->_getBcApi();
        $res = $bcApi->sendAuthenticatedPostRequest($apiEndpoint, $data);

        return $res;
    }


    /**
     * Edit plan draft to include pre-post commands & save plan.
     *
     * @param string $draftId
     */
    public function editPrePostCommands($draftId)
    {
        $apiEndpoint = "/bc/api/ams/backup/plan_drafts/$draftId/options";
        $commandConfig = array(
            'workingDirectory' => '',
            'continueOnCommandError' => false,
            'commandArguments' => '{RESOURCE_ID}',
            'waitCommandComplete' => true,
            'command' => '/usr/lib/Acronis/BackupAndRecovery/webcpprecapture'
        );

        // TODO find out if there's an option to chang the schedule to daily
        $data = array(
            'prePostCommands' => array(
                'useDefaultCommands' => false,
                'postCommands' => $commandConfig,
                'preCommands' => $commandConfig,
                'usePostCommands' => true,
                'usePreCommands' => true
            )
        );

        $bcApi = $this->_getBcApi();
        $bcApi->sendAuthenticatedPutRequest($apiEndpoint, $data);
    }

    /**
     * Rename backup plan. Does not seem to be another way to name the plan.
     *
     * @param string $draftId
     * @return array
     */
    public function renameDraft($draftId)
    {
        $bcApi = $this->_getBcApi();

        $params = "/bc/api/ams/backup/plan_drafts/$draftId/parameters";
        $data = array(
            'planName' => 'webcp'
        );

        $res = $bcApi->sendAuthenticatedPutRequest($params, $data);
        return $res;
    }

    /**
     * Create backup plan.
     *
     * @param string $draftId
     * @return array
     */
    public function createPlan($draftId)
    {
        $bcApi = $this->_getBcApi();

        $data = $this->_getDraftIdConfig($draftId);
        $apiEndpoint = '/bc/api/ams/backup/plan_drafts_operations/create_plan';
        $res = $bcApi->sendAuthenticatedPostRequest($apiEndpoint, $data);
        return $res;
    }

    /**
     * Get draft id config.
     *
     * @param string $draftId
     * @return array
     */
    private function _getDraftIdConfig($draftId)
    {
        return array(
            'draftId' => $draftId
        );
    }

    /**
     * Assign existing plan to resource.
     *
     * @param string $planId
     * @param string $resourceId
     * @return array
     */
    public function assignPlan($planId, $resourceId)
    {
        $bcApi = $this->_getBcApi();

        $apiEndpoint = '/bc/api/ams/backup/plan_operations/add_resources';
        $data = $this->_getPlanData($planId, $resourceId);

        $res = $bcApi->sendAuthenticatedPostRequest($apiEndpoint, $data);
        return $res;
    }

    /**
     * Revoke backup plan.
     *
     * @param string $planId
     * @param string $resourceId
     * @return array
     */
    public function revokePlan($planId, $resourceId)
    {
        $bcApi = $this->_getBcApi();

        $apiEndpoint = '/bc/api/ams/backup/plan_operations/remove_resources';
        $data = $this->_getPlanData($planId, $resourceId);

        $res = $bcApi->sendAuthenticatedPostRequest($apiEndpoint, $data);
        return $res;
    }

    /**
     * Run backup.
     *
     * @param string $planId
     * @param string $resourceId
     * @return array
     */
    public function runBackup($planId, $resourceId)
    {
        $bcApi = $this->_getBcApi();

        $apiEndpoint = '/bc/api/ams/backup/plan_operations/run';
        $data = $this->_getPlanData($planId, $resourceId);

        $res = $bcApi->sendAuthenticatedPostRequest($apiEndpoint, $data);
        return $res;
    }

    /**
     * Get plan data configuration.
     *
     * @param string $planId
     * @param string $resourceId
     * @return array
     */
    private function _getPlanData($planId, $resourceId)
    {
        $data = array(
            'resourcesIds' => array($resourceId),
            'planId' => $planId
        );

        return $data;
    }

    /**
     * Get activities for specific device.
     *
     * @param string $resourceId
     * @return array
     */
    public function getActivities($resourceId)
    {
        $apiEndpoint = "/bc/api/ams/protected_resources?resourcesIds=$resourceId/activities";
        $bcApi = $this->_getBcApi();
        $res = $bcApi->sendAuthenticatedGetRequest($apiEndpoint);
        return $res;
    }
}
