<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Jan 27, 2020
 */
require_once(__DIR__ . '/../AcronisApi/Auth.php');
require_once(__DIR__ . '/../AcronisApi/User.php');

/**
 * Class for installing acronis backup cloud.
 *
 * @package InterWorx
 * @subpackage CLI
 */
class AcronisInstaller extends CLI
{
    const PATH_TMP_INSTALL = '/tmp/Backup_Agent_Linux_x86_64.bin';
    const CREDENTIALS_ENV = 'acroniscredentials';
    const AGENTCOMMDATADIR = '/var/lib/Acronis';

    const URL = 'url';
    const PASSWORD = 'password';
    const USERNAME = 'username';

    private $_url;
    private $_username;
    private $_password;


    /**
     * Register functions, install & uninstall.
     */
    public function registerOptArgs()
    {
        $this->_func('install', 'install backup agent');
        $this->_func('uninstall', 'uninstall backup agent');
    }

    /**
     * Uninstall acronis backup agent.
     * TODO remove any scheduled task
     */
    protected function _uninstall()
    {
        if (!self::isInstalled()) {
            $this->fail('Acronis is not installed, nothing to do.');
        }

        $this->_uninstallBackupAgent();

        $this->_clearEnvIfExists(self::CREDENTIALS_ENV);
        $this->_clearEnvIfExists(AcronisApi_Auth::AB_TOKEN_ENV);

        $this->success('Backup agent removed!');
    }

    /**
     * Uninstall backup agent.
     */
    private function _uninstallBackupAgent()
    {
        $cmd = '/usr/lib/Acronis/BackupAndRecovery/uninstall/uninstall --auto';
        $retval = 0;
        passthru($cmd, $retval);

        if ($retval !== 0) {
            $this->fail('unable to uninstall acronis backup agent...');
        }
    }

    /**
     * Clear env flag if file exists.
     *
     * @param string $envFlag
     */
    private function _clearEnvIfExists($envFlag)
    {
        $base = Ini::get(Ini::IWORX_DIR, 'base');
        if (is_file("$base/etc/env/$envFlag")) {
            IW::Env()->clearEnvFlag($envFlag);
        }
    }

    /**
     * Install Backup Agent.
     * No distro check as we don't support anything that acronis don't.
     */
    protected function _install()
    {
        $this->_makeSureNotInstalled();
        $this->_setVariables();
        $this->_checkCredentials();

        $this->_removeInstallScriptIfNeeded();
        $this->_downloadInstallScript();
        $this->_makeFileExecutable();

        $this->_runInstallationScript();
        $this->_addPrePostScripts();
        $this->_clearEnvIfExists(self::CREDENTIALS_ENV);
    }

    /**
     * Set variables, acronis credentials.
     * Must be set before installation,
     * either manually or in UI.
     */
    private function _setVariables()
    {
        $fileContent = IW::Env()->getEnvFlag(self::CREDENTIALS_ENV);

        if ($this->_noFileContent($fileContent)) {
            $this->_failCredentialsMissing();
        }

        $toArray = true;
        $credentials = json_decode($fileContent, $toArray);

        $this->_url = $credentials[self::URL];
        $this->_username = $credentials[self::USERNAME];
        $this->_setPassword($credentials);

        if ($this->_missingRequired()) {
            $this->_failCredentialsMissing();
        }

        $User = new Acronis_User($this->_username, $this->_password, $this->_url);
        $this->_authApi = new AcronisApi_Auth($User);
    }

    /**
     * fail if credentials file is missing or empty.
     *
     * @param mixed $file
     */
    private function _noFileContent($file)
    {
        // file retrieved from env will be bool if empty.
        if (is_bool($file) || empty($file)) {
            $this->_failCredentialsMissing();
        }
    }

    /**
     * Check if any of the reqired credentials are missing.
     *
     * @return boolean
     */
    private function _missingRequired()
    {
        $usernameMissing = empty($this->_username);
        $urlMissing = empty($this->_url);
        $passwordMissing = empty($this->_password);

        return $usernameMissing || $urlMissing || $passwordMissing;
    }

    /**
     * Display fail msg for missing credentials.
     */
    private function _failCredentialsMissing()
    {
        $this->fail('You have to set credentials before running this script.');
    }

    /**
     * Set password.
     * Password will not be encrypted if user manually added credentials file.
     * Activate plugin via UI to ensure proper encryption.
     *
     * @param array $credentials
     */
    private function _setPassword(array $credentials)
    {
        $this->_password = $credentials[self::PASSWORD];
        $encryptedPassword = $credentials['encryptedPassword'];

        if (isset($encryptedPassword) && empty($this->_password)) {
            $this->_password = IWorxEncrypt::decrypt($encryptedPassword);
        }
    }

    /**
     * Fail installation if already installed.
     */
    private function _makeSureNotInstalled()
    {
        if (self::isInstalled()) {
            $this->fail('acronis already installed and configured!');
        }
    }

    /**
     * Check if acronis is installed by verifying file existance.
     *
     * @return boolean
     */
    public static function isInstalled()
    {
        return is_file('/usr/sbin/acronis_mms');
    }

    /**
     * Fail installation if acronis credentials are invalid.
     */
    private function _checkCredentials()
    {
        try {
            // Do not save token in env - should only be saved if
            // iworx license is activated. Encryption is using license.
            $saveToken = false;
            $this->_authApi->authorizeUser($saveToken);
        } catch (Exception $e) {
            $this->fail('invalid credentials');
        }
    }

    /**
     * Update file permissions to allow execution.
     */
    private function _makeFileExecutable()
    {
        chmod(self::PATH_TMP_INSTALL, 0700);
        $this->_validatePermissions(self::PATH_TMP_INSTALL);
    }

    /**
     * Check if file is executable.
     *
     * @param string $file
     */
    private function _validatePermissions($file)
    {
        if (!is_executable($file)) {
            $this->fail("couldn't change file permissions...");
        }
    }

    /**
     * Delete installation script.
     */
    private function _removeInstallScriptIfNeeded()
    {
        if (is_file(self::PATH_TMP_INSTALL)) {
            unlink(self::PATH_TMP_INSTALL);
        }
    }

    /**
     * Download acronis installation file.
     * Needs acronis user specific url with login id.
     */
    private function _downloadInstallScript()
    {
        $login = $this->_authApi->getUserId();
        $scriptUrl = $this->_getDownloadScriptUrl($this->_url, $login);
        $this->_runDownloadInstallFileCommand($scriptUrl);
    }

    /**
     * Builds url for downloading acronis installation script.
     * Url & login/userid is specific to acronis account.
     *
     * @param string $login account id
     * @return string
     */
    private function _getDownloadScriptUrl($login)
    {
        $params = '/bc/api/ams/links/agents/redirect?language=multi&system=linux&architecture=64&productType=enterprise&login=';
        $url = $this->_url . $params . $login;
        return $url;
    }

    /**
     * Runs wget command to download installation file.
     *
     * @param string $scriptUrl
     */
    private function _runDownloadInstallFileCommand($scriptUrl)
    {
        $retval = 0;
        $cmd = 'wget -O ' . self::PATH_TMP_INSTALL . ' "' . $scriptUrl . '"';
        passthru($cmd, $returnValue);

        if ($retval !== 0) {
            $this->fail("Download failed... Return value: $retval");
        }
    }


    /**
     * Execute installation script.
     */
    private function _runInstallationScript()
    {
        $cmd = $this->_getInstallCommand();

        $result = [];
        $retval = 0;
        exec($cmd, $result, $retval);

        if ($retval !== 0) {
            $this->info(json_encode($result));
            $this->fail('Installation failed... return value:'  . $retval);
        }
    }

    /**
     * Get command for running the installation file.
     *
     * @return string
     */
    private function _getInstallCommand()
    {
        // --auto - for noninteractive mode
        // --strict - converts all warnings to errors
        // --rain - backup service url
        $cmd = self::PATH_TMP_INSTALL . " --auto --strict --rain $this->_url --login $this->_username --password $this->_password";
        return $cmd;
    }

    /**
     * Add pre and post capture scripts for acronis backup.
     */
    private function _addPrePostScripts()
    {
        $srcScripts = __DIR__ . '/../scripts/AgentCommData/';
        $destination = self::AGENTCOMMDATADIR . '/';

        $result = [];
        $retval = 0;

        $cmd = "cp -a $srcScripts $destination";
        exec($cmd, $result, $retval);

        if ($retval !== 0) {
            $this->info($result);
            $this->fail('something went wrong when adding the pre/post scripts... Return value: ' . $retval);
        }

        $this->_updatePrePostScriptPermissions();
    }

    /**
     * Updata pre/post script permissions.
     */
    private function _updatePrePostScriptPermissions()
    {
        $result = [];
        $retval = 0;

        $cmd = 'chmod -R 750 ' . self::AGENTCOMMDATADIR . '/AgentCommData';
        exec($cmd, $result, $retval);

        if ($retval !== 0) {
            $this->fail('failed to update file permissions... Return value: ' . $retval);
        }
    }
}
