<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Nov 26, 2019
 */

/**
 * Nodeworx Controller for Acronis integration
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class Ctrl_Nodeworx_Acronis extends Ctrl_Nodeworx_Plugin
{
    const DASHBOARD_TAB = 'dashboard';
    const BACKUP_TAB = 'backups';
    const OPERATIONS_TAB = 'operationslog';

    const ERROR_MESSAGE = 'ab_error_message';
    const ACTIVE_TAB = 'active_tab';
    const IS_AUTHORIZED = 'authorized';
    const INSTALL_BACKUP_AGENT = 'install_backup_agent';
    const AB_MESSAGE_NOT_AUTHORIZED = '##LG_ACRONIS_NOT_AUTHORIZED##';
    const AB_MESSAGE_API_FAILURE = '##LG_ACRONIS_API_FAILURE##';

    private $_authorized;
    private $_installer;
    private $_dashboard;

    /**
     * Get required permissions.
     * Only accessible to NW users if backup is set in permissions.
     *
     * @return array
     */
    public static function getRequiredPerms()
    {
        return array(Session_NodeWorx::PERM_BRESTORE);
    }

    /**
     * Is allowed as reseller.
     *
     * @param string $action
     * @return boolean
     */
    protected function _isAllowedAsReseller($action)
    {
        return false;
    }


    /**
     * Init function. Final construct will run this method.
     */
    protected function _init()
    {
        $Plugin = $this->_getPlugin();
        $this->_authorized = $Plugin->userIsAuthorized();
        $this->_installer = new Model_Nodeworx_AcronisInstaller();
        $this->_dashboard = new Model_Nodeworx_AcronisDashboard();
    }

    /**
     * Index action.
     */
    public function indexAction()
    {
        $View = $this->getView();
        $View->assign(self::IS_AUTHORIZED, $this->_authorized);

        if (!$this->_authorized) {
            $View->assign(self::ERROR_MESSAGE, self::AB_MESSAGE_NOT_AUTHORIZED);
        }

        if ($this->_authorized && !AcronisInstaller::isInstalled()) {
            $View->assign(self::INSTALL_BACKUP_AGENT, true);
        }
    }

    /**
     * Install backup agent.
     *
     * @param Form_SubmitOnly $Form
     */
    public function installBackupAgentCommitAction(Form_SubmitOnly $Form)
    {
        $success = $this->_installer->installBackupAgent();
        $result = array(
            'success' => $success
        );

        $Payload = new Payload($result);
        $this->_displayPayload($Payload);
    }

    /**
     * Uninstall backup agent.
     *
     * @param Form_SubmitOnly $Form
     */
    public function uninstallBackupAgentCommitAction(Form_SubmitOnly $Form)
    {
        $success = $this->_installer->uninstallBackupAgent();
        $result = array(
            'success' => $success
        );

        $Payload = new Payload($result);
        $this->_displayPayload($Payload);
    }

    /**
     * Get cloud storage usage data.
     *
     * @param Form_SubmitOnly $Form
     */
    public function cloudstorageCommitAction(Form_SubmitOnly $Form)
    {
        try {
            $storageData = $this->_dashboard->getStorageData();
            $Payload = $this->_getPayload($storageData);
            $this->_displayPayload($Payload);
        } catch (Exception $e) {
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }

    /**
     * Get backup plan status (enabled/disabled) and backup plan schedule.
     *
     * @param Form_SubmitOnly $Form
     */
    public function backupPlanStatusCommitAction(Form_SubmitOnly $Form)
    {
        try {
            $backupPlan = $this->_dashboard->getBackupPlan();
            $Payload = $this->_getPayload($backupPlan);
            $this->_displayPayload($Payload);
        } catch (Exception $e) {
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }


    /**
     * Display last backup status, date, and time.
     *
     * @param Form_SubmitOnly $Form
     */
    public function backupInfoCommitAction(Form_SubmitOnly $Form)
    {
        try {
            $backupInfo = $this->_dashboard->getBackupInformation();
            $Payload = $this->_getPayload($backupInfo);
            $this->_displayPayload($Payload);
        } catch (Exception $e) {
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }

    /**
     * Get redirect URL for one-time login. Link is valid for 30 seconds.
     *
     * @param Form_SubmitOnly $Form
     */
    public function redirectToBackupCloudCommitAction(Form_SubmitOnly $Form)
    {
        try {
            $redirectUrl = $this->_dashboard->getAuthorizedLoginLink();
            $payloadData = array('redirect' => $redirectUrl);
            $Payload = $this->_getPayload($payloadData);
            $this->_displayPayload($Payload);
        } catch (Exception $e) {
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }

    /**
     * Revoke backup plan.
     *
     * @param Form_SubmitOnly $Form
     */
    public function revokePlanCommitAction(Form_SubmitOnly $Form)
    {
        try {
            $result = $this->_dashboard->revokePlan();
            $Payload = $this->_getPayload($result);
            $this->_displayPayload($Payload);
        } catch (Exception $e) {
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }

    /**
     * Activate backup plan.
     *
     * @param Form_SubmitOnly $Form
     */
    public function activatePlanCommitAction(Form_SubmitOnly $Form)
    {
        try {
            $result = $this->_dashboard->activatePlan();
            $Payload = $this->_getPayload($result);
            $this->_displayPayload($Payload);
        } catch (Exception $e) {
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }

    /**
     * Run backup.
     *
     * @param Form_SubmitOnly $Form
     */
    public function runBackupCommitAction(Form_SubmitOnly $Form)
    {
        try {
            $result = $this->_dashboard->runBackup();
            $Payload = $this->_getPayload($result);
            $this->_displayPayload($Payload);
        } catch (Exception $e) {
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }

    /**
     * Get consistently formatted payload.
     *
     * @param array $data
     * @return Payload
     */
    private function _getPayload(array $data = null)
    {
        if ($data === null) {
            $data = 0;
        }
        $responseObject = array(
            'data' => $data
        );

        return new Payload($responseObject);
    }
}
