<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Mar 31, 2020
 */

/**
 * acronis Model.
 *
 * @package    InterWorx
 * @subpackage Plugin
 */
abstract class Model_Nodeworx_Acronis
{
    const CACHE_RESOURCEID = 'resourceid.acronis';
    const CACHE_PLANID = 'planid.acronis';

    private $_installer;

    /**
     * Get resource id from cache or api if cache empty.
     *
     * @return string
     */
    protected function _getResourceId()
    {
        $resourceId = $this->_getCache(self::CACHE_RESOURCEID);

        if (empty($resourceId)) {
            $resourceId = $this->_getDashboardApi()->getResourceId();
            $this->_setCache(self::CACHE_RESOURCEID, $resourceId);
        }

        return $resourceId;
    }

    /**
     * Calling it "key" bacuase first part of it must be extracted
     * in order to use in some places of the api.
     * Acronis api is referring to both instaces as resource id.
     *
     * @param string $resourceKey
     * @return string
     */
    protected function _getIdFromResourceKey($resourceKey)
    {
        $startPos = strpos($resourceKey, '.') + 1;
        $excludedPrefix = substr($resourceKey, $startPos);

        $endPos = strpos($excludedPrefix, '@');
        $result = substr($excludedPrefix, 0, $endPos);

        return $result;
    }

    /**
     * Get installer.
     *
     * @return Model_Nodeworx_AcronisInstaller
     */
    protected function _getInstaller()
    {
        if (!isset($this->_installer)) {
            $this->_installer = new Model_Nodeworx_AcronisInstaller();
        }

        return $this->_installer;
    }

    /**
     * Get plan id.
     *
     * @return string
     */
    protected function _getPlanId()
    {
        $planId = $this->_getCache(self::CACHE_PLANID);

        if (empty($planId)) {
            $installer = $this->_getInstaller();
            $planId = $installer->setBackupPlan();
        }

        return $planId;
    }

    /**
     * Get Acronis plugin
     *
     * @return Plugin
     */
    protected function _getPlugin()
    {
        return IW::PluginManager()->getPlugin('acronis');
    }


    /**
     * Get dashboard api.
     *
     * @return AcronisApi_DashboardInterface
     */
    protected function _getDashboardApi()
    {
        if (!isset($this->_dashboardApi)) {
            $this->_dashboardApi = $this->_getPlugin()->getDashboardApi();
        }

        return $this->_dashboardApi;
    }

    /**
     * Get oauth api.
     *
     * @return AcronisApi_DashboardInterface
     */
    protected function _getOauthApi()
    {
        if (!isset($this->_dashboardApi)) {
            $this->_dashboardApi = $this->_getPlugin()->getAuthApi();
        }

        return $this->_dashboardApi;
    }

    /**
     * Get cached values.
     *
     * @param string $name
     */
    protected function _getCache($name)
    {
        return IW::Cache()->get($name);
    }


    /**
     * Format cached return object to assoc array.
     *
     * @param string $cacheName
     * @return array
     */
    protected function _getCachedArray($cacheName)
    {
        $cachedObject = $this->_getCache($cacheName);

        $assoc = true;
        return json_decode(json_encode($cachedObject), $assoc);
    }

    /**
     * Set permanent cache.
     *
     * @param string  $name
     * @param mixed   $data
     * @param integer $time
     */
    protected function _setCache($name, $data, $time = 0)
    {
        IW::Cache()->set($name, $data, $time);
    }

    /**
     * Set temporary cache. Will be cleared after 10 sec.
     *
     * @param string $name
     * @param mixed  $data
     */
    protected function _setTempCache($name, $data)
    {
        $seconds = 5;
        IW::Cache()->set($name, $data, $seconds);
    }


    /**
     * Delete cache.
     *
     * @param string $name
     */
    protected function _deleteCache($name)
    {
        IW::Cache()->delete($name);
    }
}
