<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Mar 31, 2020
 */


/**
 * acronis Model.
 *
 * @package    InterWorx
 * @subpackage Plugin
 */
class Model_Nodeworx_AcronisDashboard extends Model_Nodeworx_Acronis
{
    const CACHE_DEVICE = 'device.acronis';
    const CACHE_PLANID = 'planid.acronis';
    const CACHE_PLAN = 'plan.acronis';
    const CACHE_BACKUP_STATUS = 'status.acronis';
    const CACHE_STORAGE_INFO = 'storage.acronis';
    const CACHE_AGENT = 'agent.acronis';
    const CACHE_INSTANCE_ID = 'instanceid.acronis';
    const CACHE_OPERATIONS = 'operations.acronis';

    /**
     * Get storage data.
     *
     * @return array
     */
    public function getStorageData()
    {
        $storageData = $this->_getCachedArray(self::CACHE_STORAGE_INFO);

        if (empty($storageData)) {
            $storageData = $this->_getStorageDataFromApi();
            $this->_setTempCache(self::CACHE_STORAGE_INFO, $storageData);
        }

        return $storageData;
    }


    /**
     * Get storage data from api.
     *
     * @return array
     */
    private function _getStorageDataFromApi()
    {
        $resourceKey = $this->_getResourceId();
        $resourceId = $this->_getIdFromResourceKey($resourceKey);

        $dashboardApi = $this->_getDashboardApi();

        $size = $dashboardApi->getStorageSize($resourceId);
        $gb = $this->_bytesToGb($size);

        return array(
            'size' => $gb,
        );
    }

    /**
     * Get instanceId.
     *
     * @return string
     */
    private function _getInstanceId()
    {
        $instanceId = $this->_getCache(self::CACHE_INSTANCE_ID);

        if (empty($instanceId)) {
            $dashboardApi = $this->_getDashboardApi();
            $deviceInfo = $dashboardApi->getDeviceInfo();
            $instanceId = $deviceInfo['instanceId'];
            $this->_setCache(self::CACHE_INSTANCE_ID, $instanceId);
        }

        return $instanceId;
    }

    /**
     * Converts bytes to GB and returns size with GB suffix.
     *
     * @param float $bytes
     * @return string
     */
    private function _bytesToGb($bytes)
    {
        if ($bytes === 0) {
            return '0 GB';
        }

        $units = array('B', 'KB', 'MB', 'GB', 'TB');

        $base = log($bytes) / log(1024);
        $suffix = $units[floor($base)];


        $result = pow(1024, $base - floor($base));

        $result = round($result, 1);
        return "$result $suffix";
    }

    /**
     * Get backup plan.
     *
     * @return array
     */
    public function getBackupPlan()
    {
        $backupPlan = $this->_getCachedArray(self::CACHE_PLAN);

        if (empty($backupPlan)) {
            $backupPlan = $this->_requestPlan();
            $this->_setTempCache(self::CACHE_PLAN, $backupPlan);
        }

        return $backupPlan;
    }

    /**
     * Request backup plan.
     *
     * @return array
     */
    private function _requestPlan()
    {
        $dashboardApi = $this->_getDashboardApi();
        $resourceId = $this->_getResourceId();
        $planId = $this->_getPlanId();

        try {
            $plan = $dashboardApi->getBackupPlan($planId);
            $enabled = $dashboardApi->planTargetsResource($plan, $resourceId);
            return $this->_formatBackupPlan($plan, $enabled);
        } catch (Exception $e) {
            // handle deleted plan.
            $this->_deleteCache(self::CACHE_PLANID);
            $this->_deleteCache(self::CACHE_PLAN);

            return array(
                'enabled' => false
            );

            // TODO check if connection failed and throw exception if so.
        }
    }


    /**
     * Format backup plan from api response.
     *
     * @param array   $apiResponse
     * @param boolean $enabled
     * @return array
     */
    private function _formatBackupPlan(array $apiResponse, $enabled)
    {
        if (empty($apiResponse)) {
            throw new Exception('ERROR: Api response does not include backup plan!');
        }

        $scheme = $apiResponse['scheme'];
        $schedule = $scheme['schedule'];

        $backupPlan = array(
            'type' => $schedule['type'],
            'time' => $schedule['startAt'],
            'enabled' => $enabled,
            'id' => $apiResponse['id']
        );

        return $backupPlan;
    }

    /**
     * Get backup information. Including last & next backup.
     *
     * @return array
     */
    public function getBackupInformation()
    {
        $backupPlan = $this->_getBackupPlan();
        $result = array();

        if ($backupPlan['enabled'] === true) {
            $backupStatus = $this->_getBackupStatus();
            $lastBackup = $backupStatus['lastStartTime'];
            $nextBackup =  $backupStatus['nextStartTime'];

            $result['status'] = $backupStatus['status'];
            $result['lastBackup'] = $this->_formatDate($lastBackup);
            $result['nextBackup'] = $this->_formatDate($nextBackup);
        }

        return $result;
    }




    /**
     * Get backup plan from cache or api if cache is not set.
     *
     * @return array
     */
    private function _getBackupPlan()
    {
        $backupPlan = $this->_getCachedArray(self::CACHE_PLAN);

        if (empty($backupPlan)) {
            $backupPlan = $this->_requestPlan();
            $this->_setTempCache(self::CACHE_PLAN, $backupPlan);
        }

        return $backupPlan;
    }


    /**
     * Get backup status.
     *
     * @return array
     */
    private function _getBackupStatus()
    {
        $backupStatus = $this->_getCachedArray(self::CACHE_BACKUP_STATUS);

        if (empty($backupStatus)) {
            $backupStatus = $this->_requestBackupStatus();
            $this->_setTempCache(self::CACHE_BACKUP_STATUS, $backupStatus);
        }

        return $backupStatus;
    }

    /**
     * Get backup status from api.
     *
     * @return array
     */
    private function _requestBackupStatus()
    {
        $dashboardApi = $this->_getDashboardApi();
        $resourceId = $this->_getResourceId();
        $apiResponse = $dashboardApi->getBackupStatus($resourceId);

        $statusInfo = array(
            'status' => $apiResponse['lastStatus'],
            'lastStartTime' => $apiResponse['lastStartTime'],
            'nextStartTime' => $apiResponse['nextStartTime']
        );

        return $statusInfo;
    }


    /**
     * Configure date.
     *
     * @param string $date
     * @return array
     */
    private function _formatDate($date)
    {
        if (empty($date)) {
            return 0;
        }

        $time = strtotime($date);

        $weekday = date('l', $time);
        $day = date('d', $time);
        $month = date('F', $time);
        $year = date('Y', $time);
        $hour = date('h', $time);
        $minutes = date('i', $time);

        return array(
            'year' => $year,
            'month' => $month,
            'day' => $day,
            'weekday' => $weekday,
            'time' => "$hour:$minutes"
        );
    }


    /**
     * Get acronis login link - valid for 30 sec.
     *
     * @return string
     */
    public function getAuthorizedLoginLink()
    {
        $dashboardApi = $this->_getDashboardApi();

        $redirectUrl = $dashboardApi->getLoginUrl();
        return $redirectUrl;
    }

    /**
     * Revoke plan.
     *
     * @return array
     */
    public function revokePlan()
    {
        $dashboardApi = $this->_getDashboardApi();
        $planId = $this->_getPlanId();
        $resourceId = $this->_getResourceId();

        try {
            $dashboardApi->revokePlan($planId, $resourceId);

            $this->_deletePlanStatusCache();

            $expectEnabled = false;
            $this->_awaitUpdatedBackupStatus($expectEnabled);

            return array(
                'enabled' => false,
                'success' => true
            );
        } catch (Exception $e) {
            return array(
                'enabled' => true,
                'success' => false
            );
        }
    }

    /**
     * Activate plan.
     *
     * @return array
     */
    public function activatePlan()
    {
        $dashboardApi = $this->_getDashboardApi();
        $planId = $this->_getPlanId();
        $resourceId = $this->_getResourceId();

        try {
            $dashboardApi->assignPlan($planId, $resourceId);

            $this->_deletePlanStatusCache();

            $expectEnabled = true;
            $this->_awaitUpdatedBackupStatus($expectEnabled);
            $this->_awaitNextBackup();

            return array(
                'enabled' => true,
                'success' => true
            );
        } catch (Exception $e) {
            return array(
                'enabled' => false,
                'success' => false
            );
        }
    }

    /**
     * Delete plan & status cache.
     */
    private function _deletePlanStatusCache()
    {
        $this->_deleteCache(self::CACHE_PLAN);
        $this->_deleteCache(self::CACHE_BACKUP_STATUS);
    }

    /**
     * Send api requests until we get the expected value.
     *
     * Making sure that the API is updated before we return result,
     * so that other api requests wont show incorrect values.
     *
     * @param string $expectedValue
     */
    private function _awaitUpdatedBackupStatus($expectedValue)
    {
        $attempts = 0;

        while ($this->_planStatusNotUpdated($expectedValue) && $attempts < 10) {
            sleep(3);
            $attempts++;
        }
    }

    /**
     * Check if acronis api has updated plan status according to expected value.
     *
     * @param string $expectedValue
     */
    private function _planStatusNotUpdated($expectedValue)
    {
        try {
            $plan = $this->_requestPlan();
            return $plan['enabled'] !== $expectedValue;
        } catch (Exception $e) {
            // if plan has not been added to api yet.
            return true;
        }
    }

    /**
     * Await acronis api update for nextBackup
     * after activating plan.
     */
    private function _awaitNextBackup()
    {
        $attempts = 0;
        while ($this->_nextBackupNotUpdated() && $attempts < 10) {
            sleep(3);
            $attempts++;
        }
    }

    /**
     * Sends api request to check if next backup has been scheduled yet.
     *
     * @return boolean
     */
    private function _nextBackupNotUpdated()
    {
        $backupStatus = $this->_requestBackupStatus();
        return $backupStatus['nextStartTime'] === null;
    }

    /**
     * Run backup for entire server.
     *
     * @return array
     */
    public function runBackup()
    {
        $planId = $this->_getPlanId();
        $resourceId = $this->_getResourceId();

        $dashboardApi = $this->_getDashboardApi();
        $result = array();

        try {
            $dashboardApi->runBackup($planId, $resourceId);

            $this->_deleteCache(self::CACHE_BACKUP_STATUS);
            $this->_awaitBackupStatusRunning();

            $result['success'] = true;
        } catch (Exception $e) {
            $result['success'] = false;
        }

        return $result;
    }

    /**
     * Await acronis api update.
     * Backup status should be set to 'running' when backup is triggered.
     */
    private function _awaitBackupStatusRunning()
    {
        $attempts = 0;
        while (!$this->_backupIsRunning() && $attempts < 10) {
            sleep(2);
            $attempts++;
        }
    }

    /**
     * Check if backup status indicates that it's running.
     *
     * @return boolean
     */
    private function _backupIsRunning()
    {
        $res = $this->_requestBackupStatus();
        $backupStatus = $res['status'];
        return $backupStatus === 'running';
    }
}
