<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Mar 31, 2020
 */

/**
 * acronis Model.
 *
 * @package    InterWorx
 * @subpackage Plugin
 */
class Model_Nodeworx_AcronisInstaller extends Model_Nodeworx_Acronis
{

    /**
     * Install backup agent.
     * Returns true if successful otherwise false.
     *
     * @return boolean
     */
    public function installBackupAgent()
    {
        try {
            $this->_setEnvAcronisCredentials();
            $this->_runInstaller('--install');
            $this->setBackupPlan();
            return true;
        } catch (Exception $e) {
            $message = $e->getMessage();
            IW::log('ERROR: Failed to activate backup agent: ' . $message);
            return false;
        }
    }

    /**
     * Uninstall backup agent.
     *
     * @return boolean
     */
    public function uninstallBackupAgent()
    {
        try {
            $this->_runInstaller('--uninstall');
            return true;
        } catch (Exception $e) {
            IW::log('ERROR: ' . $e->getMessage());
            return false;
        }
    }


    /**
     * Set env acroniscredentials. Required for installation.
     */
    private function _setEnvAcronisCredentials()
    {
        $User = $this->_getPlugin()->getAcronisUser();

        $env = array(
            'username' => $User->getUsername(),
            'password' => $User->getPassword(),
            'url' => $User->getDatacenterUrl()
        );

        IW::Env()->setEnvFlag(AcronisInstaller::CREDENTIALS_ENV, json_encode($env));
    }

    /**
     * Run backup agent installation script.
     *
     * @param string $flag
     */
    private function _runInstaller($flag)
    {
        $result = [];
        $retval = -1;
        $cmd = "/home/interworx/bin/php /home/interworx/plugins/acronis/bin/acronisinstaller.php $flag";
        IWorxExec::exec($cmd, $result, $retval);

        if ($retval !== 0) {
            throw new Exception('ERROR: Error Installing Backup Agent');
        }

        if ($flag === '--install') {
            $this->_cacheResourceId();
        }
    }


    /**
     * Enable backup action.
     *
     * @return string
     */
    public function setBackupPlan()
    {
        $dashboardApi = $this->_getDashboardApi();
        $resourceId = $this->_getResourceId();
        $plans = $dashboardApi->getBackupPlans($resourceId);
        $plan = $this->_getApplicablePlan($plans);

        if (empty($plan)) {
            $plan = $this->_addBackupPlanToResource();
        }

        $planId = $plan['id'] ? $plan['id'] : $plan['planId'];

        if (empty($planId)) {
            throw new Exception('ERROR: Unable to add backup plan!');
        }

        $this->_setCache(self::CACHE_PLANID, $planId);
        return $planId;
    }

    /**
     * Get a plan with prepost commands enabled.
     *
     * @param array $plans
     * @return array
     */
    private function _getApplicablePlan(array $plans)
    {
        foreach ($plans as $plan) {
            if ($this->_prePostScriptsOK($plan)) {
                return $plan;
            }
        }
    }

    /**
     * Add backup plan to resource.
     * Will add existing plan if pre/post scripts are enabled,
     * otherwise a new plan will be created.
     *
     * @return array
     */
    protected function _addBackupPlanToResource()
    {
        $dashboardApi = $this->_getDashboardApi();
        $resourceId = $this->_getResourceId();

        $applicablePlansData = $dashboardApi->getApplicablePlans($resourceId);
        $applicablePlans = $applicablePlansData['Plans'];

        $applicablePlanId = $this->_getApplicablePlanId($applicablePlans);
        $canCreateNew = $applicablePlansData['CanCreateNew'];

        if (isset($applicablePlanId)) {
            $plan = $this->_assignPlan($applicablePlanId, $resourceId);
            return $this->_formatPlan($plan);
        }

        if ($canCreateNew) {
            $plan = $this->createNewPlan($resourceId);
            return $this->_formatPlan($plan);
        }

        if (!$canCreateNew) {
            throw new Exception('ERROR: User does not have permission to create backup plan!');
        }

        throw new Exception('ERROR: No plan was assigned!');
    }


    /**
     * Get plan id of applicable plan.
     * Will only return plan id if pre/post scripts are enabled.
     *
     * @param array $plans
     * @return string
     */
    private function _getApplicablePlanId(array $plans)
    {
        $dashboardApi = $this->_getDashboardApi();
        foreach ($plans as $plan) {
            $bcPlan = $dashboardApi->getBackupPlan($plan['ID']);
            if ($this->_prePostScriptsOK($bcPlan)) {
                return $bcPlan['id'];
            }
        }
    }


    /**
     * Check if plan has pre/post scripts enabled.
     *
     * @param array $plan
     * @return boolean
     */
    private function _prePostScriptsOK(array $plan)
    {

        $options = $plan['options'];
        $backupOptions = $options['backupOptions'];
        $prePostCommands = $backupOptions['prePostCommands'];
        $preCommand = $prePostCommands['preCommands'];
        $postCommand = $prePostCommands['postCommands'];

        $commandsEnabled = $this->_prePostCommandsEnabled($prePostCommands);
        $preCommandConfigOK = $this->_prePostCommandConfigOK($preCommand);
        $postCommandConfigOK = $this->_prePostCommandConfigOK($postCommand);

        return $commandsEnabled && $preCommandConfigOK && $postCommandConfigOK;
    }

    /**
     * Check if pre/post commands are enabled.
     *
     * @param array $commands
     * @return boolean
     */
    private function _prePostCommandsEnabled(array $commands)
    {
        $preCommandEnabled = $commands['usePreCommands'] === true;
        $postCommandEnabled = $commands['usePostCommands'] === true;

        return $preCommandEnabled && $postCommandEnabled;
    }

    /**
     * Check if existing plan has right configuration.
     *
     * @param array $commandConfig
     * @return boolean
     */
    private function _prePostCommandConfigOK(array $commandConfig)
    {
        $expectedConfig = array(
            'workingDirectory' => '',
            'continueOnCommandError' => false,
            'commandArguments' => '{RESOURCE_ID}',
            'waitCommandComplete' => true,
            'command' => '/usr/lib/Acronis/BackupAndRecovery/webcpprecapture'
        );

        foreach ($expectedConfig as $key => $value) {
            if ($commandConfig[$key] !== $value) {
                return false;
            }
        }

        return true;
    }



    /**
     * Assign existing plan to device.
     *
     * @param string $planId
     * @param string $resourceId
     * @return array
     */
    private function _assignPlan($planId, $resourceId)
    {
        $dashboardApi = $this->_getDashboardApi();
        $plan = $dashboardApi->assignPlan($planId, $resourceId);
        return $plan;
    }


    /**
     * Format plan information.
     *
     * @param array $plan
     * @return array
     */
    private function _formatPlan(array $plan)
    {
        $planId = $plan['PlanID'] ? $plan['PlanID'] : $plan['planId'];

        return array(
            'planId' => $planId
        );
    }

    /**
     * Create new backup plan.
     *
     * @param string $resourceId
     * @return array
     */
    public function createNewPlan($resourceId)
    {
        $dashboardApi = $this->_getDashboardApi();

        $draft = $dashboardApi->createDraft($resourceId);
        $draftId = $draft['DraftID'];

        $dashboardApi->renameDraft($draftId);
        $dashboardApi->editPrePostCommands($draftId); // might need to send in subid..

        $plan = $dashboardApi->createPlan($draftId);

        return $plan;
    }

    /**
     * Cache resourceId.
     */
    private function _cacheResourceId()
    {
        $dashboardApi = $this->_getDashboardApi();
        $resourceId = $dashboardApi->getResourceId();

        $this->_setCache(self::CACHE_RESOURCEID, $resourceId);
    }
}
