<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@interworx.com>
 * @date    Nov 26, 2019
 */

require_once('/home/interworx/plugins/acronis/AcronisApi/Auth.php');
require_once('/home/interworx/plugins/acronis/AcronisApi/DashboardInterface.php');
require_once('/home/interworx/plugins/acronis/AcronisApi/Dashboard.php');
require_once('/home/interworx/plugins/acronis/AcronisApi/User.php');
require_once('/home/interworx/plugins/acronis/CLI/AcronisInstaller.php');

/**
 * Acronis integration plugin.
 *
 * @package InterWorx
 * @subpackage Plugin_Acronis
 */
class Plugin_Acronis extends Plugin
{
    const FORM_SECTION_NAME = 'Acronis Account Credentials';
    const INPUT_USERNAME = 'acronis_username';
    const INPUT_PASSWORD = 'acronis_password';
    const INPUT_ACRONIS_URL = 'acronis_url';

    const AB_USERNAME = 'ab_username';
    const AB_PASSWORD = 'ab_password';
    const AB_CREDENTIALS = 'ab_credentials';
    const AB_TENANT_ID = 'ab_tenant_id';
    const AB_USER_ID = 'ab_user_id';
    const AB_URL = 'ab_url';
    const AB_ARRAY_DEVICE_INFO = 'ab_deviceinfo';

    const ACRONIS_ELEMENT_ID='acronis';
    const ACRONIS_DASHBOARD_ELEMENT_ID='acronis-dashboard';
    const ACRONIS_ELEMENT_CLASS='iw-i-plugin';

    private $_authApi;
    private $_dashboardApi;
    /**
     * Decides which category to display plugin under.
     *
     * @return string
     */
    public function getCategory()
    {
        return Plugin_Category::BACKUP_RESTORE;
    }

    /**
     * Initialize edit form.
     *
     * @param Form_NW_Plugins_Edit $Form
     */
    public function initializeMyEditForm(Form_NW_Plugins_Edit $Form)
    {
        // TODO move this to form?
        $Group = $Form->addGroup(self::AB_CREDENTIALS, self::FORM_SECTION_NAME);

        $username = new Input_String(self::INPUT_USERNAME);
        $password = new Input_Password(self::INPUT_PASSWORD);
        $url = new Input_String(self::INPUT_ACRONIS_URL);

        $Group->addInput($username)->setLabel('Username')->setRequired(false);
        $Group->addInput($password)->setLabel('Password')->setRequired(false);
        $Group->addInput($url)->setLabel('Url')->setRequired(false);
    }

    /**
     * Validates acronis credentials & installation status before processMyEditForm.
     *
     * @param Form $Form
     * @return boolean
     */
    public function validateMyEditForm(Form $Form)
    {
        // only validate if user is enabling plugin
        if ($this->_enableRequest($Form)) {
            try {
                $this->_authorizeUser($Form);
                return true;
            } catch (Exception $e) {
                $Form->addError($e->getMessage());
                return false;
            }
        }

        return true;
    }

    /**
     * Attempts to authorize user based on form credentials.
     *
     * @param Form $Form
     * @throws Exception
     */
    private function _authorizeUser(Form $Form)
    {
        $url = $Form->getValue(self::INPUT_ACRONIS_URL);
        $username = $Form->getValue(self::INPUT_USERNAME);
        $password = $Form->getValue(self::INPUT_PASSWORD);

        $User = new Acronis_User($username, $password, $url);
        $this->_authApi = new AcronisApi_Auth($User);

        // will throw exception if credentials are wrong
        $this->_authApi->authorizeUser();
    }

    /**
     * Process form.
     *
     * @param Form_NW_Plugins_Edit $Form
     */
    public function processMyEditForm(Form_NW_Plugins_Edit $Form)
    {
        // only process if plugin is being enabled.
        if ($this->_enableRequest($Form)) {
            $password = $Form->getValue(self::INPUT_PASSWORD);

            $encryptedPassword = IWorxEncrypt::encrypt($password);
            $username = $Form->getValue(self::INPUT_USERNAME);
            $tenantId = $this->_authApi->getTenantId();
            $url = $Form->getValue(self::INPUT_ACRONIS_URL);

            $this->setDatastoreVar(self::AB_PASSWORD, $encryptedPassword);
            $this->setDatastoreVar(self::AB_USERNAME, $username);
            $this->setDatastoreVar(self::AB_TENANT_ID, $tenantId);
            $this->setDatastoreVar(self::AB_URL, $url);
        }
    }

    /**
     * Check if user is attempting to disable plugin.
     *
     * @param Form $Form
     * @return boolean
     */
    private function _enableRequest(Form $Form)
    {
        return $Form->getValue('status') === '1';
    }

    /**
     * Add acronis to NW menu when enabled.
     *
     * @param IWorxMenuManager $MenuMan
     */
    public function updateNodeworxMenu(IWorxMenuManager $MenuMan)
    {
        $newMenuItemConfig = array(
            'text' => 'Acronis Backup',
            'url' => '/nodeworx/acronis',
            'class' => self::ACRONIS_ELEMENT_CLASS,
            'parent' => 'iw-menu-server',
            'disabled_for_reseller' => '1',
            'perms' => "BRESTORE"
        );

        $addAfterElementId = 'iw-menu-server-ips';
        $newElementId = self::ACRONIS_ELEMENT_ID;

        $MenuMan->addMenuItemAfter($addAfterElementId, $newElementId, $newMenuItemConfig);
    }

    /**
     * Return acronis auth api.
     *
     * @return AcronisApi_AuthInterface
     */
    public function getAuthApi()
    {
        if (!isset($this->_authApi)) {
            $User = $this->getAcronisUser();
            $this->_authApi = new AcronisApi_Auth($User);

            if (!$this->_authApi->isAuthorized()) {
                $this->_authApi->authorizeUser();
            }
        }
        return $this->_authApi;
    }

    /**
     * Get dashboard api.
     *
     * @return AcronisApi_Dashboard
     */
    public function getDashboardApi()
    {
        if (!isset($this->_dashboardApi)) {
            $authApi = $this->getAuthApi();
            $this->_dashboardApi = new AcronisApi_Dashboard($authApi);
        }

        return $this->_dashboardApi;
    }

    /**
     * Set auth api. Used in tests.
     *
     * @param AcronisApi_AuthInterface $Api
     */
    public function setAuthApi(AcronisApi_AuthInterface $Api)
    {
        $this->_authApi = $Api;
    }

    /**
     * Set dashboard api. Used in tests.
     *
     * @param AcronisApi_DashboardInterface $Api
     */
    public function setDashboardApi(AcronisApi_DashboardInterface $Api)
    {
        $this->_dashboardApi = $Api;
    }

    /**
     * Get Acronis user.
     *
     * @return Acronis_User
     */
    public function getAcronisUser()
    {
        $tenantId = $this->_getTenantId();
        $username = $this->_getUsername();
        $password = $this->_getDecryptedPassword();
        $url = $this->_getUrl();

        $User = new Acronis_User($username, $password, $url);
        $User->setTenantId($tenantId);
        return $User;
    }

    /**
     * Check if credentials are saved & validate through acronis api.
     *
     * @return boolean
     */
    public function userIsAuthorized()
    {
        $usernameNotSet = empty($this->_getUsername());
        $passwordNotSet = empty($this->_getPassword());
        $urlNotSet = empty($this->_getUrl());

        if ($usernameNotSet || $passwordNotSet || $urlNotSet) {
            return false;
        }

        $authApi = $this->getAuthApi();
        return $authApi->isAuthorized();
    }

    /**
     * Get username from datastoreVar.
     *
     * @return string
     */
    private function _getUsername()
    {
        return $this->getDatastoreVar(self::AB_USERNAME);
    }


    /**
     * Get password from datastoreVar.
     *
     * @return string
     */
    private function _getPassword()
    {
        return $this->getDatastoreVar(self::AB_PASSWORD);
    }


    /**
     * Get decrypted password from datastoreVar.
     *
     * @return string
     */
    private function _getDecryptedPassword()
    {
        $psw = $this->_getPassword();
        return IWorxEncrypt::decrypt($psw);
    }


    /**
     * Get url from datastoreVar.
     *
     * @return string
     */
    private function _getUrl()
    {
        return $this->getDatastoreVar(self::AB_URL);
    }

    /**
     * Get tenant id from datastoreVar.
     *
     * @return string
     */
    private function _getTenantId()
    {
        return $this->getDatastoreVar(self::AB_TENANT_ID);
    }
}
