<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Feb 17, 2020
 */

require_once(__DIR__ . '/../mocks/AuthMock.php');

class Ctrl_Nodeworx_AcronisTest extends IWorx_Framework_TestCase
{
    const VIEW_ERROR_MSG = 'ab_error_message';
    const VIEW_INSTALL_AGENT = 'install_backup_agent';
    const EXPECTED_MESSAGE_NOT_AUTHORIZED = '##LG_ACRONIS_NOT_AUTHORIZED##';
    const CTRL = 'Ctrl_Nodeworx_Acronis';


    private $_acronisMmsFile = '/usr/sbin/acronis_mms';

    /**
     * index action - should display error message when user not authorized.
     */
    public function testIndexActionShouldShowMessageWhenNotAuthorized()
    {
        $isAuthorized = false;
        $Reply = $this->_getReplyIndexAction($isAuthorized);

        $this->assertReplySuccessful($Reply, 'Expected successful reply from index action when not authorized.');

        // check that message was assigned to view.
        $actual = $Reply->getData()[self::VIEW_ERROR_MSG];
        $expected = self::EXPECTED_MESSAGE_NOT_AUTHORIZED;
        $this->assertSame($expected, $actual);
    }


    /**
     * Index action should retun successful reply for authorized user &
     * show button to enable backup when  backup agent is not installed
     *
     * ONLY ACCURATE IF BACKUP AGENT HAS NOT BEEN INSTALLED ALREADY.
     * Run tests on a server without the backup agent installed.
     */
    public function testIndexActionShouldEnableButtonWhenNotInstalled()
    {

        // TODO check for button
        $isAuthorized = true;
        $Reply = $this->_getReplyIndexAction($isAuthorized);

        $this->assertReplySuccessful($Reply, 'Expected successful reply from index action when authorized.');

        // check that view gets should install agent.
        $expected = true;
        $actual = $Reply->getData()[self::VIEW_INSTALL_AGENT];
        $this->assertSame($expected, $actual);
    }


    /**
     * index action - should not display any error messages.
     */
    public function testIndexActionShouldNotShowAnyErrorMessage()
    {
        // similates backup agent installation
        // by adding fake file.
        $this->_similateBackupAgentInstall();

        $isAuthorized = true;
        $Reply = $this->_getReplyIndexAction($isAuthorized);

        $this->assertReplySuccessful($Reply, 'Expected successful reply from index action when backup agent is installed.');
        $this->assertArrayNotHasKey(self::VIEW_ERROR_MSG, $Reply->getData());
    }

    /**
     * Add empty mms file. Path is used in the controller to determine
     * whether or not a backup agent has been installed.
     */
    private function _similateBackupAgentInstall()
    {
        if (!is_file($this->_acronisMmsFile)) {
            $installationFile = fopen($this->_acronisMmsFile, "w") or die("Unable to open file!");
            fclose($installationFile);
        }
    }

    /**
     * Send request to index action.
     *
     * @param boolean $isAuthorized
     * @return Reply
     */
    private function _getReplyIndexAction($isAuthorized)
    {
        // make sure mock api is used in controller.
        $Api = new AcronisApi_AuthMock($isAuthorized);
        IW::PluginManager()->getPlugin('acronis')->setAuthApi($Api);

        $NW = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'index',
            array(),
            $NW
        );

        return $Reply;
    }

    /**
     * Cleanup after each test.
     * Make sure that the mms file is removed if not the actual one.
     * Used in the controller to determine whether or not the backup agent is installed.
     */
    public function tearDown()
    {
        if (is_file($this->_acronisMmsFile)) {
            $this->_removeMmsIfEmpty();
        }
    }

    /**
     * Remove mms file if it's empty/ added for test purposes.
     */
    private function _removeMmsIfEmpty()
    {
        $fileContent = file_get_contents($this->_acronisMmsFile);
        if (empty($fileContent)) {
            unlink($this->_acronisMmsFile);
        }
    }
}
