<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Mar 27, 2020
 */

require_once(__DIR__ . '/../mocks/AuthMock.php');
require_once(__DIR__ . '/../mocks/DashboardMock.php');

//TODO extend test controllers - they have same functions, e.g. tearDown and setUp.
class Ctrl_Nodeworx_AcronisTest extends IWorx_Framework_TestCase
{
    const CTRL = 'Ctrl_Nodeworx_Acronis';
    const ACRONIS_MMS_PATH = '/usr/sbin/acronis_mms';
    const PLAN_ID = 'someplanid';
    const OK_WITH_PAYLOAD = "OK with payload";

    /**
     * Set up before each test.
     */
    public function setUp()
    {
        $this->_similateBackupAgentInstall();
        $this->_setDashboardApiMock();

        // set temp cache so that planId exists when testing.
        // would have to mock plan assignment otherwise.
        IW::Cache()->set(Model_Nodeworx_Acronis::CACHE_PLANID, self::PLAN_ID, 30);
    }

    /**
     * set dashboard api mock.
     */
    private function _setDashboardApiMock()
    {
        $DashboardApiMock = new AcronisApi_DashboardMock();
        IW::PluginManager()->getPlugin('acronis')->setDashboardApi($DashboardApiMock);
    }


    /**
     * Add empty mms file. Path is used in the controller to determine
     * whether or not a backup agent has been installed.
     */
    private function _similateBackupAgentInstall()
    {
        if (!is_file(self::ACRONIS_MMS_PATH)) {
            $installationFile = fopen(self::ACRONIS_MMS_PATH, "w") or die("Unable to open file!");
            fclose($installationFile);
        }
    }

    /**
     * Cloudstorage commit action. Should display payload.
     */
    public function testCloudstorageCommitActionTokenMissing()
    {
        $action = 'cloudstorageCommit';
        $Reply = $this->_getReply($action);

        $msg = 'cloudstorageCommitAction should return successful reply';
        $this->assertReplySuccessful($Reply, $msg);

        $expected = REPLY::OK_PAYLOAD;
        $actual = $Reply->getCode();

        $this->assertSame($expected, $actual);
    }

    /**
     *  test backup commit action. should display payload.
     */
    public function testBackupCommitAction()
    {
        $action = 'backupPlanStatusCommit';
        $Reply = $this->_getReply($action);

        $msg = 'backupCommitAction should return successful reply';
        $this->assertReplySuccessful($Reply, $msg);

        $expected = REPLY::OK_PAYLOAD;
        $actual = $Reply->getCode();

        $this->assertSame($expected, $actual);
    }

    /**
     *  test last backup commit action. should display payload.
     */
    public function testBackupInfoCommitAction()
    {
        $action = 'backupInfoCommit';
        $Reply = $this->_getReply($action);

        $msg = 'lastBackupCommitAction should return successful reply';
        $this->assertReplySuccessful($Reply, $msg);

        $expected = REPLY::OK_PAYLOAD;
        $actual = $Reply->getCode();

        $this->assertSame($expected, $actual);
    }

    /**
     * Send request to controller.
     *
     * @param string $action
     * @return Reply
     */
    private function _getReply($action)
    {
        $isAuthorized = true;
        $Api = new AcronisApi_AuthMock($isAuthorized);
        IW::PluginManager()->getPlugin('acronis')->setAuthApi($Api);

        $NW = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            $action,
            array(),
            $NW
        );

        return $Reply;
    }

    /**
     * Cleanup after each test.
     * Make sure that the mms file is removed if not the actual one.
     * Used in the controller to determine whether or not the backup agent is installed.
     */
    public function tearDown()
    {
        if (is_file(self::ACRONIS_MMS_PATH)) {
            $this->_removeMmsIfEmpty();
        }

        // TODO delete all cache
    }

    /**
     * Remove mms file if it's empty/ added for test purposes.
     */
    private function _removeMmsIfEmpty()
    {
        $fileContent = file_get_contents(self::ACRONIS_MMS_PATH);
        if (empty($fileContent)) {
            unlink($this->_acronisMmsFile);
        }
    }
}
